@@ -4,6 +4,7 @@ from __future__ import absolute_import, division, print_function
 import functools
 import os
 from threading import Thread
+from datetime import timedelta
 
 from lostc import collection as lcc
 
@@ -20,16 +21,16 @@ from suanpan.utils import pbar as spbar
 
 class Storage(base.Storage):
     def __init__(
-        self,
-        minioAccessKey=None,
-        minioSecretKey=None,
-        minioBucketName="suanpan",
-        minioEndpoint="minio-serivce.default:9000",
-        minioSecure=True,
-        minioDelimiter="/",
-        minioTempStore=base.Storage.DEFAULT_TEMP_DATA_STORE,
-        minioGlobalStore=base.Storage.DEFAULT_GLOBAL_DATA_STORE,
-        **kwargs,
+            self,
+            minioAccessKey=None,
+            minioSecretKey=None,
+            minioBucketName="suanpan",
+            minioEndpoint="minio-serivce.default:9000",
+            minioSecure=True,
+            minioDelimiter="/",
+            minioTempStore=base.Storage.DEFAULT_TEMP_DATA_STORE,
+            minioGlobalStore=base.Storage.DEFAULT_GLOBAL_DATA_STORE,
+            **kwargs,
     ):
         super().__init__(
             delimiter=minioDelimiter,
@@ -46,11 +47,11 @@ class Storage(base.Storage):
     def _analyzeEndpoint(self, endpoint, secure=False):
         httpsPrefix = "https://"
         if endpoint.startswith(httpsPrefix):
-            return endpoint[len(httpsPrefix) :], True
+            return endpoint[len(httpsPrefix):], True
 
         httpPrefix = "http://"
         if endpoint.startswith(httpPrefix):
-            return endpoint[len(httpPrefix) :], False
+            return endpoint[len(httpPrefix):], False
 
         return endpoint, secure
 
@@ -87,13 +88,13 @@ class Storage(base.Storage):
 
     @runtime.retry(stop_max_attempt_number=3)
     def download(
-        self,
-        key,
-        path=None,
-        progress=None,
-        bucket=None,
-        ignores=None,
-        quiet=not g.debug,
+            self,
+            key,
+            path=None,
+            progress=None,
+            bucket=None,
+            ignores=None,
+            quiet=not g.debug,
     ):
         ignores = ignores or self.DEFAULT_IGNORE_KEYWORDS
         downloadFunction = (
@@ -106,14 +107,14 @@ class Storage(base.Storage):
         )
 
     def downloadFolder(
-        self,
-        key,
-        path=None,
-        progress=None,
-        bucket=None,
-        delimiter=None,
-        ignores=None,
-        quiet=not g.debug,
+            self,
+            key,
+            path=None,
+            progress=None,
+            bucket=None,
+            delimiter=None,
+            ignores=None,
+            quiet=not g.debug,
     ):
         bucket = bucket or self.bucket
         ignores = ignores or self.DEFAULT_IGNORE_KEYWORDS
@@ -126,7 +127,7 @@ class Storage(base.Storage):
 
         if self.basename(key) in ignores:
             with spbar.one(
-                total=1, desc="Downloading", disable=quiet, progress=progress
+                    total=1, desc="Downloading", disable=quiet, progress=progress
             ) as pbar:
                 pbar.update(1)
                 pbar.set_description("Ignored")
@@ -166,13 +167,13 @@ class Storage(base.Storage):
         return path
 
     def downloadFile(
-        self,
-        key,
-        path=None,
-        progress=None,
-        bucket=None,
-        ignores=None,
-        quiet=not g.debug,
+            self,
+            key,
+            path=None,
+            progress=None,
+            bucket=None,
+            ignores=None,
+            quiet=not g.debug,
     ):
         bucket = bucket or self.bucket
         ignores = ignores or self.DEFAULT_IGNORE_KEYWORDS
@@ -184,7 +185,7 @@ class Storage(base.Storage):
             logger.info(f"Downloading file: {url} -> {path}")
 
         with spbar.one(
-            total=fileSize, desc="Downloading", disable=quiet, progress=progress
+                total=fileSize, desc="Downloading", disable=quiet, progress=progress
         ) as pbar:
             if self.basename(key) in ignores:
                 pbar.update(fileSize)
@@ -208,13 +209,13 @@ class Storage(base.Storage):
 
     @runtime.retry(stop_max_attempt_number=3)
     def upload(
-        self,
-        name,
-        path=None,
-        progress=None,
-        bucket=None,
-        ignores=None,
-        quiet=not g.debug,
+            self,
+            name,
+            path=None,
+            progress=None,
+            bucket=None,
+            ignores=None,
+            quiet=not g.debug,
     ):
         bucket = bucket or self.bucket
         ignores = ignores or self.DEFAULT_IGNORE_KEYWORDS
@@ -224,13 +225,13 @@ class Storage(base.Storage):
         )
 
     def uploadFolder(
-        self,
-        key,
-        path=None,
-        progress=None,
-        bucket=None,
-        ignores=None,
-        quiet=not g.debug,
+            self,
+            key,
+            path=None,
+            progress=None,
+            bucket=None,
+            ignores=None,
+            quiet=not g.debug,
     ):
         bucket = bucket or self.bucket
         ignores = ignores or self.DEFAULT_IGNORE_KEYWORDS
@@ -242,7 +243,7 @@ class Storage(base.Storage):
 
         if self.basename(key) in ignores:
             with spbar.one(
-                total=1, desc="Uploading", disable=quiet, progress=progress
+                    total=1, desc="Uploading", disable=quiet, progress=progress
             ) as pbar:
                 pbar.update(1)
                 pbar.set_description("Ignored")
@@ -289,13 +290,13 @@ class Storage(base.Storage):
         return path
 
     def uploadFile(
-        self,
-        key,
-        path=None,
-        progress=None,
-        bucket=None,
-        ignores=None,
-        quiet=not g.debug,
+            self,
+            key,
+            path=None,
+            progress=None,
+            bucket=None,
+            ignores=None,
+            quiet=not g.debug,
     ):
         bucket = bucket or self.bucket
         ignores = ignores or self.DEFAULT_IGNORE_KEYWORDS
@@ -307,7 +308,7 @@ class Storage(base.Storage):
             logger.info(f"Uploading file: {path} -> {url}")
 
         with spbar.one(
-            total=fileSize, desc="Uploading", disable=quiet, progress=progress
+                total=fileSize, desc="Uploading", disable=quiet, progress=progress
         ) as pbar:
             if self.basename(key) in ignores:
                 pbar.update(fileSize)
@@ -341,7 +342,7 @@ class Storage(base.Storage):
         return copyFunction(src, dist, progress=progress, bucket=bucket, quiet=quiet)
 
     def copyFolder(
-        self, src, dist, progress=None, bucket=None, delimiter=None, quiet=not g.debug,
+            self, src, dist, progress=None, bucket=None, delimiter=None, quiet=not g.debug,
     ):
         bucket = bucket or self.bucket
         delimiter = delimiter or self.delimiter
@@ -374,7 +375,7 @@ class Storage(base.Storage):
             logger.info(f"Copying file: {srcurl} -> {disturl}")
 
         with spbar.one(
-            total=fileSize, desc="Copying", disable=quiet, progress=progress
+                total=fileSize, desc="Copying", disable=quiet, progress=progress
         ) as pbar:
             objectMd5 = self.getStorageMd5(src, bucket=bucket)
             distMd5 = self.getStorageMd5(dist, bucket=bucket)
@@ -391,7 +392,7 @@ class Storage(base.Storage):
 
     @runtime.retry(stop_max_attempt_number=3)
     def remove(
-        self, key, progress=None, delimiter=None, bucket=None, quiet=not g.debug
+            self, key, progress=None, delimiter=None, bucket=None, quiet=not g.debug
     ):
         delimiter = delimiter or self.delimiter
         bucket = bucket or self.bucket
@@ -401,7 +402,7 @@ class Storage(base.Storage):
         return removeFunc(key, delimiter=delimiter, bucket=bucket, quiet=quiet)
 
     def removeFolder(
-        self, key, progress=None, delimiter=None, bucket=None, quiet=not g.debug
+            self, key, progress=None, delimiter=None, bucket=None, quiet=not g.debug
     ):
         delimiter = delimiter or self.delimiter
         bucket = bucket or self.bucket
@@ -428,7 +429,7 @@ class Storage(base.Storage):
         return key
 
     def removeFile(
-        self, key, progress=None, delimiter=None, bucket=None, quiet=not g.debug
+            self, key, progress=None, delimiter=None, bucket=None, quiet=not g.debug
     ):  # pylint: disable=unused-argument
         delimiter = delimiter or self.delimiter
         bucket = bucket or self.bucket
@@ -438,7 +439,7 @@ class Storage(base.Storage):
 
         _remove = self.autoRefreshToken(self.client.remove_object)
         with spbar.one(
-            total=1, disable=quiet, desc="Removing", progress=progress,
+                total=1, disable=quiet, desc="Removing", progress=progress,
         ) as pbar:
             self.autoRefreshToken(self.client.remove_object)(bucket, key)
             pbar.update(1)
@@ -485,6 +486,16 @@ class Storage(base.Storage):
                 return False
             raise e
 
+    def presignedGetObject(self, key, expires=3600, bucket=None):
+        bucket = bucket or self.bucket
+        url = self.client.presigned_get_object(bucket, key, expires=timedelta(seconds=expires))
+        return url
+
+    def presignedPutObject(self, key, expires=3600, bucket=None):
+        bucket = bucket or self.bucket
+        url = self.client.presigned_put_object(bucket, key, expires=timedelta(seconds=expires))
+        return url
+
     def getStorageMd5(self, key, bucket=None):
         bucket = bucket or self.bucket
         try:
